@@ -187,7 +187,7 @@ class MySQLHelper(object):
             cursor.close()
 
     def execute(self, sql):
-        """Execute arbitary SQL against the database."""
+        """Execute arbitrary SQL against the database."""
         cursor = self.connection.cursor()
         try:
             cursor.execute(sql)
@@ -729,7 +729,7 @@ class MySQL8Helper(MySQLHelper):
                 remote_ip=remote_ip,
                 password=password)
             )
-        except MySQLdb._exceptions.OperationalError:
+        except MySQLdb.OperationalError:
             log("DB user {} already exists.".format(db_user),
                 "WARNING")
         finally:
