@@ -1,4 +1,4 @@
-# Copyright 2014-2015 Canonical Limited.
+# Copyright 2014-2021 Canonical Limited.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,9 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-#
-# Copyright 2012 Canonical Ltd.
-#
 # This file is sourced from lp:openstack-charm-helpers
 #
 # Authors:
@@ -297,7 +294,6 @@ class BasePool(object):
         # NOTE: Do not perform initialization steps that require live data from
         # a running cluster here. The *Pool classes may be used for validation.
         self.service = service
-        self.nautilus_or_later = cmp_pkgrevno('ceph-common', '14.2.0') >= 0
         self.op = op or {}
 
         if op:
@@ -344,7 +340,8 @@ class BasePool(object):
         Do not add calls for a specific pool type here, those should go into
         one of the pool specific classes.
         """
-        if self.nautilus_or_later:
+        nautilus_or_later = cmp_pkgrevno('ceph-common', '14.2.0') >= 0
+        if nautilus_or_later:
             # Ensure we set the expected pool ratio
             update_pool(
                 client=self.service,
@@ -605,7 +602,7 @@ class BasePool(object):
 
 
 class Pool(BasePool):
-    """Compability shim for any descendents external to this library."""
+    """Compatibility shim for any descendents external to this library."""
 
     @deprecate(
         'The ``Pool`` baseclass has been replaced by ``BasePool`` class.')
@@ -663,8 +660,9 @@ class ReplicatedPool(BasePool):
         else:
             self.pg_num = self.get_pgs(self.replicas, self.percent_data)
 
+        nautilus_or_later = cmp_pkgrevno('ceph-common', '14.2.0') >= 0
         # Create it
-        if self.nautilus_or_later:
+        if nautilus_or_later:
             cmd = [
                 'ceph', '--id', self.service, 'osd', 'pool', 'create',
                 '--pg-num-min={}'.format(
@@ -748,9 +746,9 @@ class ErasurePool(BasePool):
         k = int(erasure_profile['k'])
         m = int(erasure_profile['m'])
         pgs = self.get_pgs(k + m, self.percent_data)
-        self.nautilus_or_later = cmp_pkgrevno('ceph-common', '14.2.0') >= 0
+        nautilus_or_later = cmp_pkgrevno('ceph-common', '14.2.0') >= 0
         # Create it
-        if self.nautilus_or_later:
+        if nautilus_or_later:
             cmd = [
                 'ceph', '--id', self.service, 'osd', 'pool', 'create',
                 '--pg-num-min={}'.format(
@@ -1535,7 +1533,7 @@ def map_block_storage(service, pool, image):
 
 
 def filesystem_mounted(fs):
-    """Determine whether a filesytems is already mounted."""
+    """Determine whether a filesystem is already mounted."""
     return fs in [f for f, m in mounts()]
 
 
@@ -1904,7 +1902,7 @@ class CephBrokerRq(object):
                                 set the ceph-mon unit handling the broker
                                 request will set its default value.
         :type erasure_profile: str
-        :param allow_ec_overwrites: allow EC pools to be overriden
+        :param allow_ec_overwrites: allow EC pools to be overridden
         :type allow_ec_overwrites: bool
         :raises: AssertionError if provided data is of invalid type/range
         """
@@ -1949,7 +1947,7 @@ class CephBrokerRq(object):
         :param lrc_locality: Group the coding and data chunks into sets of size locality
                              (lrc plugin)
         :type lrc_locality: int
-        :param durability_estimator: The number of parity chuncks each of which includes
+        :param durability_estimator: The number of parity chunks each of which includes
                                      a data chunk in its calculation range (shec plugin)
         :type durability_estimator: int
         :param helper_chunks: The number of helper chunks to use for recovery operations
@@ -2327,7 +2325,7 @@ class CephOSDConfContext(CephConfContext):
     settings are in conf['osd_from_client'] and finally settings which do
     clash are in conf['osd_from_client_conflict']. Rather than silently drop
     the conflicting settings they are provided in the context so they can be
-    rendered commented out to give some visability to the admin.
+    rendered commented out to give some visibility to the admin.
     """
 
     def __init__(self, permitted_sections=None):
