@@ -1164,6 +1164,14 @@ class HelpersTest(TestCase):
         self.assertEqual(result['foo'], 'BAR')
         check_output.assert_called_with(['relation-get', '--format=json', '-'])
 
+        hookenv.relation_get(unit='foo/0')
+        check_output.assert_called_with(['relation-get', '--format=json', '-', 'foo/0'])
+
+        hookenv.relation_get(app='foo')
+        check_output.assert_called_with(['relation-get', '--format=json', '--app', '-', 'foo'])
+
+        self.assertRaises(ValueError, hookenv.relation_get, app='foo', unit='foo/0')
+
     @patch('charmhelpers.core.hookenv.subprocess')
     def test_relation_get_none(self, mock_subprocess):
         mock_subprocess.check_output.return_value = b'null'
@@ -1256,6 +1264,9 @@ class HelpersTest(TestCase):
         hookenv.relation_set(foo="bar")
         check_call_.assert_called_with(['relation-set', 'foo=bar'])
 
+        hookenv.relation_set(foo="bar", app=True)
+        check_call_.assert_called_with(['relation-set', '--app', 'foo=bar'])
+
     @patch('charmhelpers.core.hookenv.local_unit')
     @patch('subprocess.check_output')
     @patch('subprocess.check_call')
@@ -1670,6 +1681,15 @@ class HelpersTest(TestCase):
                                  any_order=True)
         self.assertEqual(expected_settings, proxy_settings)
 
+    @patch.object(hookenv, 'metadata')
+    def test_is_subordinate(self, mock_metadata):
+        mock_metadata.return_value = {}
+        self.assertFalse(hookenv.is_subordinate())
+        mock_metadata.return_value = {'subordinate': False}
+        self.assertFalse(hookenv.is_subordinate())
+        mock_metadata.return_value = {'subordinate': True}
+        self.assertTrue(hookenv.is_subordinate())
+
 
 class HooksTest(TestCase):
     def setUp(self):
@@ -1767,10 +1787,10 @@ class HooksTest(TestCase):
         self.assertEqual(execs, [True])
 
     def test_magic_underscores(self):
-        # Juju hook names use hypens as separators. Python functions use
+        # Juju hook names use hyphens as separators. Python functions use
         # underscores. If explicit names have not been provided, hooks
         # are registered with both the function name and the function
-        # name with underscores replaced with hypens for convenience.
+        # name with underscores replaced with hyphens for convenience.
         execs = []
         hooks = hookenv.Hooks()
 
