@@ -121,13 +121,13 @@ class Test_apt_pkg_Cache(unittest.TestCase):
             },
         }
         self.assertEquals(
-            apt_cache._dpkg_list(['package']), expect)
+            apt_cache.dpkg_list(['package']), expect)
         self.check_output.side_effect = subprocess.CalledProcessError(
             1, '', output=self.check_output.return_value)
-        self.assertEquals(apt_cache._dpkg_list(['package']), expect)
+        self.assertEquals(apt_cache.dpkg_list(['package']), expect)
         self.check_output.side_effect = subprocess.CalledProcessError(2, '')
         with self.assertRaises(subprocess.CalledProcessError):
-            _ = apt_cache._dpkg_list(['package'])
+            _ = apt_cache.dpkg_list(['package'])
 
     def test_version_compare(self):
         self.patch_object(apt_pkg.subprocess, 'check_call')
